<?php
defined('ABSPATH') || exit;

class SparxpresWebSaleAdmin {
	private $plugin;

	/**
	 * Constructor
	 * @param $version
	 * @param $mainFile
	 */
	public function __construct($version, $mainFile) {
		// Plugin Details
        $this->plugin               = new stdClass;
        $this->plugin->name         = 'sparxpres-websale-plugin'; // Plugin Folder
        $this->plugin->displayName  = 'Sparxpres for WooCommerce'; // Plugin Name
        $this->plugin->version      = $version;
        $this->plugin->folder       = plugin_dir_path( $mainFile );
        $this->plugin->url          = plugin_dir_url( $mainFile );

        // Hooks
		add_action( 'admin_init', array( $this, 'registerSettings' ) );
        add_action( 'admin_menu', array( $this, 'addSubmenuPage' ) );

        // Filters
		add_filter( 'plugin_action_links_' . plugin_basename( $mainFile ), array( $this, 'add_settings_link' ) );
	}
	
	/**
	* Register Settings
	*/
	function registerSettings() {
		register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_LINK_ID, 'trim' );
        register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_VIEW_TYPE, 'trim' );
        register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_PRODUCT_PAGE, 'trim' );
		register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_CART_PAGE, 'trim' );
		register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_CONTENT_DISPLAY_TYPE, 'trim' );
		register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_INFO_PAGE_ID, 'intval' );
		register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_MAIN_COLOR, 'trim' );
        register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_SLIDER_BG_COLOR, 'trim' );
		register_setting( $this->plugin->name, SparxpresUtils::$DK_SPARXPRES_CALLBACK_IDENTIFIER, 'trim' );
	}

    /**
     * Cleanup old order statuses
     */
    function updateOldOrderStatuses() {
        if (!get_option('dk_spx_old_order_status_converted')) {
            global $wpdb;
            $wpdb->get_results("UPDATE " . $wpdb->prefix . "posts SET post_status = 'wc-completed' WHERE post_type = 'shop_order' AND post_status = 'wc-spx-captured'");
            $wpdb->get_results("UPDATE " . $wpdb->prefix . "posts SET post_status = 'wc-pending' WHERE post_type = 'shop_order' AND post_status = 'wc-spx-processing'");
            add_option('dk_spx_old_order_status_converted', true);
            delete_option('dk_spx_convert_old_order_status');
        }
    }

    /**
     * Send callback key to Sparxpres
     */
    function sendCallbackKey() {
        if (SparxpresUtils::get_link_id() && !get_option('dk_spx_callback_key_send')) {
            if (SparxpresUtils::post_callback_key()) {
                $this->message = __('Callback key are sent to Sparxpres.', 'sparxpres-websale-plugin');
                add_option('dk_spx_callback_key_send', true);
            }
        }
    }

    /**
    * Register the plugin settings panel
    */
    function addSubmenuPage() {
    	add_submenu_page( 'options-general.php', $this->plugin->displayName, $this->plugin->displayName, 'manage_options', $this->plugin->name, array( $this, 'adminPanel' ) );
	}

	/**
	 * Settings link
	 * @param $links
	 * @return array
	 */
	function add_settings_link( $links ) {
		$settings_link = '<a href="' . admin_url( 'options-general.php?page=' . $this->plugin->name ) . '">' . __('Settings') . '</a>';
		array_unshift( $links, $settings_link );
		return $links;
	}

	/**
    * Output the Administration Panel
    * Save POSTed data from the Administration Panel into a WordPress option
    */
    function adminPanel() {
		// only admin user can access this page
		if ( !current_user_can( 'administrator' ) ) {
			echo '<p>' . __( 'Sorry, you are not allowed to access this page.', 'sparxpres-websale-plugin' ) . '</p>';
			return;
		}

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            // Save Settings?
            if (isset($_REQUEST['btnSave'])) {
                // Check nonce
                if (!isset($_REQUEST[$this->plugin->name . '_nonce'])) {
                    // Missing nonce
                    $this->errorMessage = __('nonce field is missing. Settings NOT saved.', 'sparxpres-websale-plugin');
                } elseif (!wp_verify_nonce($_REQUEST[$this->plugin->name . '_nonce'], $this->plugin->name)) {
                    // Invalid nonce
                    $this->errorMessage = __('Invalid nonce specified. Settings NOT saved.', 'sparxpres-websale-plugin');
                } else {
                    try {
                        // Save
                        update_option(SparxpresUtils::$DK_SPARXPRES_LINK_ID, trim(sanitize_key($_REQUEST[SparxpresUtils::$DK_SPARXPRES_LINK_ID])));
                        update_option(SparxpresUtils::$DK_SPARXPRES_VIEW_TYPE, sanitize_key($_REQUEST[SparxpresUtils::$DK_SPARXPRES_VIEW_TYPE]));
                        update_option(SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_PRODUCT_PAGE, sanitize_key($_REQUEST[SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_PRODUCT_PAGE]));
                        update_option(SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_CART_PAGE, sanitize_key($_REQUEST[SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_CART_PAGE]));
                        update_option(SparxpresUtils::$DK_SPARXPRES_CONTENT_DISPLAY_TYPE, sanitize_key($_REQUEST[SparxpresUtils::$DK_SPARXPRES_CONTENT_DISPLAY_TYPE]));
                        if (isset($_REQUEST[SparxpresUtils::$DK_SPARXPRES_INFO_PAGE_ID])) {
                            update_option(SparxpresUtils::$DK_SPARXPRES_INFO_PAGE_ID, sanitize_key(intval($_REQUEST[SparxpresUtils::$DK_SPARXPRES_INFO_PAGE_ID])));
                        }
                        update_option(SparxpresUtils::$DK_SPARXPRES_MAIN_COLOR, sanitize_hex_color($_REQUEST[SparxpresUtils::$DK_SPARXPRES_MAIN_COLOR]));
                        update_option(SparxpresUtils::$DK_SPARXPRES_SLIDER_BG_COLOR, sanitize_hex_color($_REQUEST[SparxpresUtils::$DK_SPARXPRES_SLIDER_BG_COLOR]));
                        $this->message = __('Settings Saved.', 'sparxpres-websale-plugin');

                        // Update old order statuses
                        $this->updateOldOrderStatuses();
                        // Send callback key
                        $this->sendCallbackKey();
                    } catch(Exception $e) {
                        $this->errorMessage = __('Settings NOT saved. Caught exception: ' . $e->getMessage(), 'sparxpres-websale-plugin');
                    }
                }
            }
        }

        // Get latest settings
        $this->settings = array(
			SparxpresUtils::$DK_SPARXPRES_LINK_ID => esc_html( wp_unslash( get_option( SparxpresUtils::$DK_SPARXPRES_LINK_ID ) ) ),
            SparxpresUtils::$DK_SPARXPRES_VIEW_TYPE => esc_html( wp_unslash( get_option( SparxpresUtils::$DK_SPARXPRES_VIEW_TYPE ) ) ),
            SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_PRODUCT_PAGE => esc_html( wp_unslash( get_option( SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_PRODUCT_PAGE ) ) ),
			SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_CART_PAGE => esc_html( wp_unslash( get_option( SparxpresUtils::$DK_SPARXPRES_WRAPPER_TYPE_CART_PAGE ) ) ),
			SparxpresUtils::$DK_SPARXPRES_CONTENT_DISPLAY_TYPE => esc_html( wp_unslash( get_option( SparxpresUtils::$DK_SPARXPRES_CONTENT_DISPLAY_TYPE ) ) ),
			SparxpresUtils::$DK_SPARXPRES_INFO_PAGE_ID => intval( get_option( SparxpresUtils::$DK_SPARXPRES_INFO_PAGE_ID ) ),
			SparxpresUtils::$DK_SPARXPRES_MAIN_COLOR => esc_html( wp_unslash( get_option( SparxpresUtils::$DK_SPARXPRES_MAIN_COLOR ) ) ),
            SparxpresUtils::$DK_SPARXPRES_SLIDER_BG_COLOR => esc_html( wp_unslash( get_option( SparxpresUtils::$DK_SPARXPRES_SLIDER_BG_COLOR ) ) ),
			SparxpresUtils::$DK_SPARXPRES_CALLBACK_IDENTIFIER => SparxpresUtils::get_callback_identifier(),
        );

    	// Load Settings Form
        include_once( $this->plugin->folder . '/includes/admin/sparxpres-admin-settings.php' );
    }
}
