<?php
/**
 * Plugin Name: Sparxpres for WooCommerce
 * Version:     1.2.10
 * Author: 		Sparxpres
 * Author URI:  https://sparxpres.dk/
 * Description: Integrates Sparxpres payment gateway into your WooCommerce installation.
 * License:     GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 */

defined('ABSPATH') || exit;

/**
 * Activation and deactivation hooks for WordPress
 */
function sparxpres_for_woocommerce_activate() {
    // activation logic goes here.
}
register_activation_hook( __FILE__, 'sparxpres_for_woocommerce_activate' );

function sparxpres_for_woocommerce_deactivate() {
    // deactivation logic goes here.
}
register_deactivation_hook( __FILE__, 'sparxpres_for_woocommerce_deactivate' );


if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
    if ( !class_exists('WC_Sparxpres_Web_Sale')) {

        class WC_Sparxpres_Web_Sale {
            /**
             * The single instance of the class.
             */
            private static $_instance = null;

            /**
             * Static values
             */
            private static $_version = '1.2.10';
            private static $_mainFile = __FILE__;

            /**
             * Constructor.
             */
            protected function __construct() {
                include_once dirname(self::$_mainFile) . '/includes/sparxpres-utils.php';
                include_once dirname(self::$_mainFile) . '/includes/payment-gateway/sparxpres-payment-gateway.php';
                include_once dirname(self::$_mainFile) . '/includes/payment-gateway/xprespay-payment-gateway.php';
            }

            /**
             * Main Extension Instance.
             * Ensures only one instance of the extension is loaded or can be loaded.
             */
            public static function instance() {
                if ( is_null( self::$_instance ) ) {
                    self::$_instance = new self();
                }

                return self::$_instance;
            }

            /**
             * Cloning is forbidden.
             */
            public function __clone() {
                // Override this PHP function to prevent unwanted copies of your instance.
                wc_doing_it_wrong( __FUNCTION__, __( 'Cloning is forbidden.', 'woocommerce' ), '2.1' );
            }

            /**
             * Unserializing instances of this class is forbidden.
             */
            public function __wakeup() {
                // Override this PHP function to prevent unwanted copies of your instance.
                wc_doing_it_wrong( __FUNCTION__, __( 'Unserializing instances of this class is forbidden.', 'woocommerce' ), '2.1' );
            }

            /**
             * @return SparxpresWebSaleAdmin|SparxpresWebSaleFrontend|void
             */
            public function run() {
                if ( is_admin() ) {
                    if (!class_exists('SparxpresWebSaleAdmin', false)) {
                        include_once dirname(self::$_mainFile) . '/includes/admin/sparxpres-admin.php';
                        return new SparxpresWebSaleAdmin(self::$_version, self::$_mainFile);
                    }
                } else {
                    if (!class_exists('SparxpresWebSaleFrontend', false)) {
                        include_once dirname(self::$_mainFile) . '/includes/rest/callback.php';
                        include_once dirname(self::$_mainFile) . '/includes/frontend/sparxpres-frontend.php';
                        return new SparxpresWebSaleFrontend(self::$_version, self::$_mainFile);
                    }
                }
            }
        }
    }

    WC_Sparxpres_Web_Sale::instance()->run();
}
